# pylint: disable=W0622
"""cubicweb-prov application packaging information"""

modname = 'prov'
distname = 'cubicweb-prov'

numversion = (0, 5, 0)
version = '.'.join(str(num) for num in numversion)

license = 'LGPL'
author = 'LOGILAB S.A. (Paris, FRANCE)'
author_email = 'contact@logilab.fr'
description = 'The PROV Ontology'
web = 'http://www.cubicweb.org/project/%s' % distname

__depends__ = {'cubicweb': ">= 3.24.0, < 3.32.0"}
__recommends__ = {}

classifiers = [
    'Environment :: Web Environment',
    'Framework :: CubicWeb',
    'Programming Language :: Python',
    'Programming Language :: JavaScript',
]



# copyright 2015-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-prov entity's classes"""

from cubicweb.entities import AnyEntity, fetch_config
from cubicweb.view import EntityAdapter
from cubicweb.predicates import relation_possible


class Activity(AnyEntity):
    __regid__ = 'Activity'
    fetch_attrs, cw_fetch_order = fetch_config(('start', 'end', 'type',
                                                'description', 'description_format'),
                                               order='DESC')


class IRecordable(EntityAdapter):
    """Adapter for entities whose changes should be recorded using prov's Activity"""
    __regid__ = 'IRecordable'
    __select__ = relation_possible('generated', role='object')

    def add_activity(self, **kwargs):
        """Return a new Activity with `generated` and `used` relations on this Entity.
        """
        kwargs.setdefault('used', self.entity)
        kwargs.setdefault('generated', self.entity)
        return self._cw.create_entity('Activity', **kwargs)



# copyright 2015-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-prov schema"""

from yams.buildobjs import EntityType, RelationType, String, RichString, TZDatetime
from yams.constraints import BoundaryConstraint, Attribute

from cubicweb import _


class Activity(EntityType):
    """An activity is something that occurs over a period of time and acts
    upon or with entities; it may include consuming, processing, transforming,
    modifying, relocating, using, or generating entities.
    """
    # See http://www.w3.org/TR/2013/NOTE-prov-dc-20130430/#prov-refinements
    type = String(internationalizable=True,
                  vocabulary=[_('publish'), _('contribute'), _('create'),
                              _('right_assignement'), _('modify'),
                              _('accept'), _('copyright'), _('submit'),
                              _('replace')])
    description = RichString(fulltextindexed=True)
    start = TZDatetime(constraints=[BoundaryConstraint('<=', Attribute('end'))],
                       description=_('when the activity started'))
    end = TZDatetime(constraints=[BoundaryConstraint('>=', Attribute('start'))],
                     description=_('when the activity ended'))


class associated_with(RelationType):
    """An activity association is an assignment of responsibility to an agent
    for an activity, indicating that the agent had a role in the activity. It
    further allows for a plan to be specified, which is the plan intended by
    the agent to achieve some goals in the context of this activity.
    """
    subject = 'Activity'
    cardinality = '?*'
    description = _('the agent responsible for this activity')


class generated(RelationType):
    """Relate an Activity to an other entity, which is the product of the
    activity.
    """
    subject = 'Activity'
    cardinality = '?*'
    inlined = True
    description = _('completion of production of a new entity by an activity')
    composite = 'object'


class used(RelationType):
    """Relate an Activity to an other entity, which is used as the source of
    the activity (usually for the production of an other entity through the
    `generated` relation).
    """
    subject = 'Activity'
    cardinality = '?*'
    inlined = True
    description = _('beginning of utilizing an entity by an activity')
    composite = 'object'


class derived_from(RelationType):
    """Relate an Entity to an other entity, through a process of derivation
    which can be a transformation of an entity into another, an update of an
    entity resulting in a new one, or the construction of a new entity based
    on a pre-existing entity.
    """
    description = _('entity used in a derivation process resulting in another '
                    'entity or a modification of the entity')



# copyright 2015-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-prov site customizations"""

from cubicweb.xy import xy


xy.register_prefix('prov', 'http://www.w3.org/ns/prov#')

xy.add_equivalence('Activity', 'prov:Activity')
xy.add_equivalence('Activity type', 'prov:type')
xy.add_equivalence('Activity description', 'prov:label')
xy.add_equivalence('Activity start', 'prov:startedAtTime')
xy.add_equivalence('Activity end', 'prov:endedAtTime')
xy.add_equivalence('Activity associated_with', 'prov:wasAssociatedWith')
xy.add_equivalence('Activity generated', 'prov:generated')
xy.add_equivalence('Activity used', 'prov:used')



# copyright 2015-2022 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
# contact http://www.logilab.fr -- mailto:contact@logilab.fr
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU Lesser General Public License as published by the Free
# Software Foundation, either version 2.1 of the License, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
# details.
#
# You should have received a copy of the GNU Lesser General Public License along
# with this program. If not, see <http://www.gnu.org/licenses/>.
"""cubicweb-prov views/forms/actions/components for web ui"""

from cubicweb import _, tags
from cubicweb.predicates import relation_possible, has_related_entities, is_instance
from cubicweb.view import EntityView
from cubicweb.web.views import uicfg, tableview


class ActivityGeneratedView(EntityView):
    """View for Activity, to be displayed in the context of the entity generated by the activity.
    """
    __regid__ = 'prov.activity-generated'
    __select__ = relation_possible('generated', 'object')
    empty_msg = _('no activity')
    rql = ('Any E,T,DE,DEF,D,A ORDERBY D DESC '
           'WHERE E generated X, E associated_with A?, E type T, E start D,'
           '      E description DE, E description_format DEF, X eid %(x)s')

    def entity_call(self, entity, **kwargs):
        rset = self._cw.execute(self.rql, {'x': entity.eid})
        self.w(tags.h2(self._cw._('generated_object')))
        if rset:
            self._cw.view('prov.activity-generated.table', rset, w=self.w)
        else:
            self.w(tags.div(self._cw._('no activity'), klass="text-muted"))


class ActivityGeneratedTable(tableview.EntityTableView):
    __regid__ = 'prov.activity-generated.table'
    __select__ = is_instance('Activity')
    columns = ['type', 'description', 'start', 'agent']
    column_renderers = {
        'agent': tableview.RelatedEntityColRenderer(
            getrelated=lambda x: x.associated_with and x.associated_with[0] or None),
    }


class ActivityAssociatedWithView(EntityView):
    """View for Activity, to be displayed in the context of the Agent associated with the Activity.
    """
    __regid__ = 'prov.activity-associated-with'
    __select__ = has_related_entities('associated_with', 'object')

    def entity_call(self, entity, **kwargs):
        rset = self._cw.execute(
            'Any E,T,DE,DEF,D,A ORDERBY D DESC WHERE '
            'E associated_with X, E generated A, E type T, E start D,'
            'E description DE, E description_format DEF, X eid %(x)s', {'x': entity.eid})
        self._cw.view('prov.activity-associated-with.table', rset, w=self.w)


class ActivityAssociatedWithTable(tableview.EntityTableView):
    __regid__ = 'prov.activity-associated-with.table'
    __select__ = is_instance('Activity')
    columns = ['type', 'description', 'start', 'generated']
    column_renderers = {
        'generated': tableview.RelatedEntityColRenderer(
            getrelated=lambda x: x.generated[0]),
    }


def registration_callback(vreg):
    vreg.register_all(globals().values(), __name__)

    afs = uicfg.autoform_section
    pvs = uicfg.primaryview_section
    abaa = uicfg.actionbox_appearsin_addmenu

    for rtype in ('used', 'generated', 'associated_with'):
        if rtype in vreg.schema:
            pvs.tag_object_of(('*', rtype, '*'), 'hidden')
            afs.tag_object_of(('*', rtype, '*'), 'main', 'hidden')
            abaa.tag_object_of(('*', rtype, '*'), False)

    if 'derived_from' in vreg.schema:
        afs.tag_subject_of(('*', 'derived_from', '*'), 'main', 'hidden')
        afs.tag_object_of(('*', 'derived_from', '*'), 'main', 'hidden')
